window.REACT_APP_KEYCLOAK_REALM = undefined
window.REACT_APP_KEYCLOAK_URL = undefined
window.REACT_APP_KEYCLOAK_CLIENT_ID = undefined
window.REACT_APP_STORAGE_TYPE = undefined
window.REACT_APP_DOMAIN_BASE = undefined
window.REACT_APP_PROJECT_DISPLAY_NAME = undefined
window.REACT_APP_GRAFANA_DASHBOARD_URL = undefined
