# Constants for relation field types
FOREIGN_KEY_NAME = 'ForeignKey'
MANY_TO_MANY_NAME = 'ManyToManyField'
ONE_TO_ONE_NAME = 'OneToOneField'
RELATION_FIELD_TYPES = {FOREIGN_KEY_NAME, MANY_TO_MANY_NAME, ONE_TO_ONE_NAME}